Package: mCSEA
Type: Package
Title: Methylated CpGs Set Enrichment Analysis
Version: 1.32.0
Author: Jordi Martorell-Marugán and Pedro Carmona-Sáez
Maintainer: Jordi Martorell-Marugán <jmartorellm@gmail.com>
Description: Identification of diferentially methylated regions (DMRs) in predefined regions
    (promoters, CpG islands...) from the human genome using Illumina's 450K or EPIC microarray data.
    Provides methods to rank CpG probes based on linear models and includes plotting functions.
Depends: R (>= 3.5), mCSEAdata, Homo.sapiens
Suggests: Biobase, BiocGenerics, BiocStyle, FlowSorted.Blood.450k,
        knitr, leukemiasEset, minfi, minfiData, rmarkdown, RUnit
Imports: biomaRt, fgsea, GenomicFeatures, GenomicRanges, ggplot2,
        graphics, grDevices, Gviz, IRanges, limma, methods, parallel,
        S4Vectors, stats, SummarizedExperiment, utils
VignetteBuilder: knitr
biocViews: ImmunoOncology, DifferentialMethylation, DNAMethylation,
        Epigenetics, Genetics, GenomeAnnotation, MethylationArray,
        Microarray, MultipleComparison, TwoChannel
License: GPL-2
Encoding: UTF-8
LazyData: true
git_url: https://git.bioconductor.org/packages/mCSEA
git_branch: RELEASE_3_23
git_last_commit: 05a0cc5
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-06-03
NeedsCompilation: no
Packaged: 2026-06-04 01:02:48 UTC; biocbuild
Built: R 4.6.0; ; 2026-06-04 19:04:24 UTC; unix

export(mCSEAPlot)
export(mCSEAPlotGSEA)
export(mCSEATest)
export(rankProbes)
export(mCSEAIntegrate)
import(mCSEAdata)
import(Homo.sapiens)
importFrom("grDevices", "dev.off", "pdf")
importFrom("stats", "cor.test", "lm", "model.matrix",
            "na.omit", "p.adjust", "relevel", "sd")
importFrom("graphics", "abline", "legend", "par", "plot")
importFrom("methods", "is")

#' Rank CpG probes
#'
#' Apply a linear model to Illumina's 450k or EPIC methylation data to get the
#' t-value of each CpG probe
#'
#' @param methData A data frame or a matrix containing Illumina's CpG probes in
#' rows and samples in columns. A SummarizedExperiment object can be used too
#' @param pheno A data frame or a matrix containing samples in rows and
#' covariates in columns. If NULL (default), pheno is extracted from the
#' SummarizedExperiment object
#' @param paired Perform a paired t-test (default = FALSE)
#' @param explanatory The column name or position from pheno used to perform the
#'  comparison between groups (default = first column)
#' @param covariates A list or character vector with column names from pheno
#' used as data covariates in the linear model
#' @param pairColumn Only for paired analysis. The column name or position from
#' pheno used to connect the paired samples (default = NULL)
#' @param caseGroup The group name or position from explanatory variable used as
#' cases to perform the comparison (default = first group)
#' @param refGroup The group name or position from explanatory variable used as
#' reference to perform the comparison (default = second group)
#' @param continuous A list or character vector with columns names from pheno
#' which should be treated as continuous variables (default = none)
#' @param typeInput Type of input methylation data. "beta" for Beta-values and
#' "M" for M-values
#' @param typeAnalysis "M" to use M-values to rank the CpG probes (default).
#' "beta" to use Beta-values instead
#'
#' @return A named vector containing the t-values from the linear model for each
#'  CpG probe
#'
#' @author Jordi Martorell Marugán, \email{jordi.martorell@@genyo.es}
#'
#' @references Smyth, G. K. (2005). \emph{Limma: linear models for microarray
#' data}. Bioinformatics and Computational Biology Solutions using R and
#' Bioconductor, 397-420.
#'
#' @seealso \code{\link{mCSEATest}}
#'
#' @examples
#' data(mcseadata)
#' myRank <- rankProbes(betaTest, phenoTest, refGroup = "Control")
#' head(myRank)
#' @export

rankProbes <- function(methData, pheno = NULL, paired = FALSE, explanatory = 1,
                    covariates = c(), pairColumn = c(), caseGroup = 1,
                    refGroup = 2, continuous = NULL,
                    typeInput = "beta", typeAnalysis = "M")
    {

    # Check input objects
    if (!any(class(methData) == "data.frame" | class(methData) == "matrix" |
            class(methData) == "SummarizedExperiment" |
            class(methData) == "RangedSummarizedExperiment")){
        stop("methData must be a data frame, a matrix or a SummarizedExperiment
            object")
    }

    if (!any(class(pheno) == "data.frame" | class(pheno) == "matrix" |
            is.null(pheno))){
        stop("pheno must be a data frame, a matrix or NULL")
    }

    if (class(paired) != "logical"){
        stop("paired must be a logical object (TRUE/FALSE)")
    }

    if (paired) {
        if (!any(class(pairColumn) != "character" |
                !is.numeric(pairColumn))){
            stop("pairColumn must be a character or numeric object")
        }
    }

    if (!any(class(explanatory) != "character" |
            !is.numeric(explanatory))){
        stop("explanatory must be a character or numeric object")
    }

    if (!any(class(covariates) == "character" | class(covariates) == "list" |
            is.null(covariates))){
        stop("covariates must be a character vector, a list or NULL")
    }

    if (!any(class(caseGroup) != "character" |
            class(caseGroup) != "numeric")){
        stop("caseGroup must be a character or numeric object")
    }

    if (!any(class(refGroup) != "character" |
            class(refGroup) != "numeric")){
        stop("refGroup must be a character or numeric object")
    }

    if (!any(class(continuous) == "character" | class(continuous) == "list" |
            is.null(continuous))){
        stop("continuous must be a character vector, a list or NULL")
    }

    if (class(typeInput) != "character"){
        stop("typeInput must be a character object")
    }

    if (class(typeAnalysis) != "character"){
        stop("typeAnalysis must be a character object")
    }

    # Get data from SummarizedExperiment objects

    if (any(class(methData) == "SummarizedExperiment" |
            class(methData) == "RangedSummarizedExperiment")){
        if (is.null(pheno)){
            pheno <- SummarizedExperiment::colData(methData)
        }
        methData <- SummarizedExperiment::assay(methData)
    }
    else {
        if (is.null(pheno)) {
            stop("If methData is not a SummarizedExperiment, you must provide
                pheno parameter")
        }
    }

    # Check that colnames of methData and rownames of pheno are both characters
    colnames(methData) = as.character(colnames(methData))
    rownames(pheno) = as.character(rownames(pheno))

    if (is.null(continuous)){
        continuous <- c()
        categorical <- colnames(pheno)
    }
    else {
        if (class(continuous) != "character") {
            continuous <-colnames(pheno)[continuous]
        }
        categorical <- setdiff(colnames(pheno), continuous)
    }

    # Ensure all categorial variables are factors, continuous variables are
    # numeric and there are no lists
    for (column in colnames(pheno)) {
        if (column %in% categorical) {
            if (typeof(pheno[,column]) == "list") {
                message(paste(column, "variable skipped due to it is a list"))
                pheno[, -which(names(pheno) == column)]
            }
            else {
                pheno[,column] <- factor(pheno[,column])
            }
        }
        else {
            pheno[,column] <- as.numeric(as.character(pheno[,column]))
        }
    }

    typeInput <- match.arg(typeInput, c("beta", "M"))
    typeAnalysis <- match.arg(typeAnalysis, c("M", "beta"))


    if (class(explanatory) == "numeric") {
        explanatory <- colnames(pheno)[explanatory]
    }

    if (is.numeric(covariates)) {
        covariates <- colnames(pheno)[covariates]
    }

    if (is.numeric(pairColumn)) {
        pairColumn <- colnames(pheno)[pairColumn]
    }

    if (class(caseGroup) == "numeric") {
        caseGroup <- levels(pheno[,explanatory])[caseGroup]
    }

    if (class(refGroup) == "numeric") {
        refGroup <- levels(pheno[,explanatory])[refGroup]
    }

    if (length(intersect(explanatory, covariates)) > 0) {
        stop("You specified some variable(s) as both explanatory and covariate")
    }



    samples2Include <- rownames(pheno)[pheno[,explanatory] %in%
                                        c(caseGroup, refGroup)]

    methData <- methData[,samples2Include]

    pheno <- data.frame(pheno[samples2Include, c(explanatory, covariates,
                                                pairColumn)])
    colnames(pheno) <- c(explanatory, covariates, pairColumn)
    pheno <- droplevels(pheno)

    # Prepare methylation data for limma
    if (typeInput == "beta") {
        if (any(min(methData, na.rm=TRUE) < 0 | max(methData, na.rm=TRUE) > 1)){
            warning("Introduced beta-values are not between 0 and 1. Are you
                    sure these are not M-values?")
        }

        if (typeAnalysis == "beta") {
            dataLimma <- methData
        }
        else {
            message("Transforming beta-values to M-values")
            dataLimma <- log2(methData) - log2(1 - methData)
        }
    }
    else {
        if (min(methData, na.rm=TRUE) >= 0 && max(methData, na.rm=TRUE) <= 1) {
            warning("Introduced M-values are between 0 and 1. Are you sure these
                    are not beta-values?")
        }

        if (typeAnalysis == "beta") {
            message("Transforming M-values to beta-values")
            dataLimma <- 2^(methData)/(1 + 2^(methData))
        }
        else {
            dataLimma <- methData
        }
    }

    # Perform linear model
    message("Calculating linear model...")
    message(paste("\tExplanatory variable:", explanatory))

    if (is.factor(pheno[,explanatory])){
        message(paste("\tCase group:", caseGroup))
        message(paste("\tReference group:", refGroup))
        message(paste("\tTotal samples:", ncol(methData)))
        pheno[,explanatory] <- relevel(pheno[,explanatory],
                                        ref=refGroup)
    }

    if (is.null(covariates) && !paired){
        message("\tCovariates: None")
        message(paste("\tCategorical variables:",
                    paste(categorical, collapse=" ")))
        if (length(continuous) > 0) {
            message(paste("\tContinuous variables:",
                            paste(continuous, collapse=" ")))
        }
        else {
            message("\tContinuous variables: None")
        }

        model <- model.matrix(~get(explanatory), data=pheno)

        }
    else {
        message(paste("\tPaired analysis using", pairColumn, "column"))
        message(paste("\tCovariates:", paste(covariates, collapse=" ")))
        message(paste("\tCategorical variables:",
                    paste(categorical, collapse=" ")))
        message(paste("\tContinuous variables:",
                    paste(continuous, collapse=" ")))
        model <- model.matrix(~., data=pheno)
    }

    linearModel <- limma::eBayes(limma::lmFit(dataLimma, model))

    tValues <- linearModel[["t"]][,2]
    return(tValues)
}

#' Plot mCSEA results
#'
#' Generate an enrichment plot
#'
#' @param rank A named numeric vector with the ranking statistic of each CpG
#' site
#' @param mCSEAResults The object generated by mCSEATest function
#' @param regionType The region type to be represented. Must be one of
#' "promoters", "genes", "CGI" or "custom"
#' @param dmrName The DMR of interest to be represented (e.g. gene name,
#' CGI name...)
#'
#' @return 'NULL'
#'
#' @author Jordi Martorell Marugán, \email{jordi.martorell@@genyo.es}
#'
#' @seealso \code{\link{rankProbes}}, \code{\link{mCSEATest}},
#' \code{\link{mCSEAPlot}}
#'
#' @references Subramanian, A. et al (2005). \emph{Gene set enrichment analysis:
#'  A knowledge-based approach for interpreting genome-wide expression profiles}
#'  . PNAS 102, 15545-15550.
#'
#' @examples
#' \dontrun{
#' library(mCSEAdata)
#' data(mcseadata)
#' myRank <- rankProbes(betaTest, phenoTest, refGroup = "Control")
#' set.seed(123)
#' myResults <- mCSEATest(myRank, regionsTypes = "promoters",
#' platform = "EPIC")
#' }
#' data(precomputedmCSEA)
#' mCSEAPlotGSEA(myRank, myResults, "promoters", "CLIC6")
#' @export


mCSEAPlotGSEA <- function(rank, mCSEAResults, regionType, dmrName){

    # Check input objects
    if (any(class(mCSEAResults) != "list" | length(mCSEAResults) < 2)){
        stop("mCSEAResults must be the object generated
            by mCSEATest function")
    }

    if (class(dmrName) != "character"){
        stop("dmrName must be a character object")
    }

    # Get the appropiate association

    regionType <- match.arg(regionType,
                            choices=c("promoters", "genes", "CGI", "custom"))

    if(regionType == "promoters") {
        assoc <- mCSEAResults[["promoters_association"]]
    }

    else if(regionType == "genes") {
        assoc <- mCSEAResults[["genes_association"]]
    }

    else if(regionType == "CGI") {
        assoc <- mCSEAResults[["CGI_association"]]
    }

    else if(regionType == "custom") {
        assoc <- mCSEAResults[["custom_association"]]
    }

    if (!dmrName %in% names(assoc)){
        stop("dmrName name is not in the provided mCSEA results")
    }

    fgsea::plotEnrichment(assoc[[dmrName]], rank) + ggplot2::labs(title=dmrName)
}

#' mCSEA core analysis
#'
#' Perform a methylated CpG sites enrichment analysis in predefined genomic
#' regions
#'
#' @param rank A named numeric vector with the ranking statistic of each CpG
#' site
#' @param methData A data frame or a matrix containing Illumina's CpG probes in
#' rows and samples in columns. A SummarizedExperiment object can be used too
#' @param pheno A data frame or a matrix containing samples in rows and
#' covariates in columns. If NULL (default), pheno is extracted from the
#' SummarizedExperiment object
#' @param column The column name or number from pheno used to split the samples
#' into groups (first column is used by default)
#' @param regionsTypes A character or character vector indicating the predefined
#'  regions to be analyzed. NULL to skip this step and use customAnnotation.
#' @param customAnnotation An optional list with the CpGs associated to each
#' feature (default = NULL)
#' @param minCpGs Minimum number of CpGs associated to a region. Regions below
#' this threshold are not tested
#' @param nproc Number of processors to use in GSEA step (default = 1)
#' @param nperm (deprecated) Number of permutations to do in GSEA step in the 
#' previous implementation. Now, this parameter is ignored
#' @param platform Platform used to get the methylation data (450k, EPIC or EPICv2)
#'
#' @return A list with the results of each of the analyzed regions. For each
#' region type, a data frame with the results and a list with the probes
#' associated to each region are generated. In addition, this list also contains
#' the input methData, pheno and platform objects
#'
#' @author Jordi Martorell Marugán, \email{jordi.martorell@@genyo.es}
#'
#' @seealso \code{\link{rankProbes}}, \code{\link{mCSEAPlot}},
#' \code{\link{mCSEAPlotGSEA}}
#'
#' @references Subramanian, A. et al (2005). \emph{Gene set enrichment analysis:
#'  A knowledge-based approach for interpreting genome-wide expression profiles}
#'  . PNAS 102, 15545-15550.
#'
#' @examples
#' \dontrun{
#' library(mCSEAdata)
#' data(mcseadata)
#' myRank <- rankProbes(betaTest, phenoTest, refGroup = "Control")
#' set.seed(123)
#' myResults <- mCSEATest(myRank, betaTest, phenoTest,
#' regionsTypes = "promoters", platform = "EPIC")
#' }
#' data(precomputedmCSEA)
#' head(myResults[["promoters"]])
#' head(myResults[["promoters_association"]])
#' @export


mCSEATest <- function(rank, methData, pheno = NULL, column = 1,
                      regionsTypes = c("promoters", "genes", "CGI"),
                      customAnnotation = NULL, minCpGs = 5, nproc = 1,
                      nperm = NULL, platform = "450k")
{
  
  output <- list()
  
  # Check input objects
  if (!any(class(rank) == "numeric" | class(rank) == "integer")){
    stop("rank must be a numeric vector")
  }
  
  if (!typeof(rank) == "double"){
    stop("rank must be a named vector")
  }
  
  if (!any(class(methData) == "data.frame" | class(methData) == "matrix" |
           class(methData) == "SummarizedExperiment" |
           class(methData) == "RangedSummarizedExperiment")){
    stop("methData must be a data frame, a matrix or a SummarizedExperiment
            object")
  }
  
  if (!any(class(pheno) == "data.frame" | class(pheno) == "matrix" |
           is.null(pheno))){
    stop("pheno must be a data frame, a matrix or NULL")
  }
  
  if (!identical(colnames(methData), rownames(pheno))) {
    if (length(setdiff(colnames(methData),  rownames(pheno))) == 0 &&
        length(setdiff( rownames(pheno), colnames(methData))) == 0) {
      pheno <- pheno[colnames(methData),]
    }
    
    else {
      stop("Sample labels of methData and pheno must be the same")
    }
  }
  
  
  if (!any(class(column) != "character" |
           !is.numeric(column))){
    stop("column must be a character or numeric object")
  }
  
  if (class(regionsTypes) != "character" & !is.null(regionsTypes)){
    stop("regionsTypes must be a character")
  }
  
  if (is.null(regionsTypes) & is.null(customAnnotation)){
    stop("Either regionsTypes or customAnnotations must be specified")
  }
  
  if (!any(class(customAnnotation) == "list" | is.null(customAnnotation))){
    stop("customAnnotation must be a list or NULL")
  }
  
  if (class(platform) != "character"){
    stop("platform must be a character object")
  }
  
  # Get data from SummarizedExperiment objects
  
  if (any(class(methData) == "SummarizedExperiment" |
          class(methData) == "RangedSummarizedExperiment")){
    if (is.null(pheno)){
      pheno <- SummarizedExperiment::colData(methData)
    }
    methData <- SummarizedExperiment::assay(methData)
  }
  else {
    if (is.null(pheno)) {
      stop("If methData is not a SummarizedExperiment, you must provide
                pheno parameter")
    }
  }
  
  
  platform <- match.arg(platform, c("450k", "EPIC", "EPICv2"))
  
  if (platform == "450k") {
    assocPromoters <- mCSEAdata::assocPromoters450k
    assocGenes <- mCSEAdata::assocGenes450k
    assocCGI <- mCSEAdata::assocCGI450k
    
  } else if (platform == "EPIC") {
    assocPromoters <- mCSEAdata::assocPromotersEPIC
    assocGenes <- mCSEAdata::assocGenesEPIC
    assocCGI <- mCSEAdata::assocCGIEPIC
  } else { ############## add EPICv2 annotations
    assocPromoters <- mCSEAdata::assocPromotersEPICv2
    assocGenes <- mCSEAdata::assocGenesEPICv2
    assocCGI <- mCSEAdata::assocCGIEPICv2
  }
  
  if (!is.null(regionsTypes)){
    regionsTypes <- match.arg(regionsTypes,
                              choices=c("promoters","genes","CGI"),
                              several.ok=TRUE)
    
    for (region in regionsTypes) {
      if (region == "promoters") {
        
        resGSEA <- .performGSEA(region, rank, platform, assocPromoters,
                                minCpGs, nproc)
        
        output[["promoters"]] <- resGSEA[[1]]
        output[["promoters_association"]] <- resGSEA[[2]]
        
      }
      
      else if (region == "genes") {
        
        resGSEA <- .performGSEA(region, rank, platform, assocGenes,
                                minCpGs, nproc)
        
        output[["genes"]] <- resGSEA[[1]]
        output[["genes_association"]] <- resGSEA[[2]]
        
      }
      else if (region == "CGI") {
        
        resGSEA <- .performGSEA(region, rank, platform, assocCGI,
                                minCpGs, nproc)
        
        output[["CGI"]] <- resGSEA[[1]]
        output[["CGI_association"]] <- resGSEA[[2]]
        
      }
    }
    
  }
  
  if (!is.null(customAnnotation)) {
    
    resGSEA <- .performGSEA("custom", rank, platform, customAnnotation,
                            minCpGs, nproc)
    
    output[["custom"]] <- resGSEA[[1]]
    output[["custom_association"]] <- resGSEA[[2]]
  }
  
  output[["methData"]] <- methData
  output[["pheno"]] <- data.frame(Group = factor(pheno[,column]),
                                  row.names = rownames(pheno))
  output[["platform"]] <- platform
  return(output)
}


.performGSEA <- function(region, rank, platform, assoc, minCpGs, nproc) {
  
  if (region != "custom"){
    message(paste("Associating CpG sites to", region))
    
    if (platform == "450k" & length(rank) > 242500 |
        platform == "EPIC" & length(rank) > 433000 |
        platform == "EPICv2" & length(rank) > 465000){
      dataDiff <- setdiff(unlist(assoc), names(rank))
      genes <- lapply(assoc,
                      function(x) {x[!x %in% dataDiff]})
    }
    
    else {
      genes <- lapply(assoc,
                      function(x) {x[x %in% names(rank)]})
    }
  }
  else {
    genes <- assoc
  }
  
  message(paste("Analysing", region))
  
  fgseaRes <- fgsea::fgseaMultilevel(genes, rank, minSize=minCpGs,
                                     nproc=nproc)
  fgseaDataFrame <- as.data.frame(fgseaRes)
  rownames(fgseaDataFrame) <- fgseaDataFrame[,1]
  fgseaDataFrame <- fgseaDataFrame[,-1]
  message(paste(sum(fgseaDataFrame[["padj"]] < 0.05),
                "DMRs found (padj < 0.05)"))
  fgseaSorted <- fgseaDataFrame[order(fgseaDataFrame[["NES"]]),]
  fgseaSorted[,7] <- vapply(fgseaSorted[,7],
                            function(x) {paste(unlist(x),
                                               collapse=", ")},
                            "")
  
  return(list(fgseaSorted, genes))
}

test_mCSEATest <- function() {
    set.seed(123)
    checkEqualsNumeric(mCSEATest(myRank[1:100], betaTest, phenoTest, 
                                    regionsTypes = "p", platform = "EPIC")[["promoters"]]["PKN3","ES"],
                    0.6470448, tolerance=1.0e-6)
}

#' Plot mCSEA results
#'
#' Generate a graphic with the genomic context of the selected DMR, showing
#' methylation status at each CpG site of different samples groups
#'
#' @param mCSEAResults The object generated by mCSEATest function
#' @param regionType The region type to be represented. Must be one of
#' "promoters", "genes", "CGI" or "custom"
#' @param dmrName The DMR of interest to be represented (e.g. gene name, CGI
#' name...)
#' @param extend The number of base pairs to extend the plot around the DMR
#' of interest (default = 1000 bp)
#' @param chromosome If TRUE, represent the chromosome and genome axis
#' @param leadingEdge If TRUE, represent the bars indicating if each CpG belongs
#'  to the mCSEA leading edge (green) or not (red)
#' @param CGI If TRUE, represent the annotated CpG islands
#' @param genes If TRUE, represent the annotated genes
#' @param transcriptAnnotation Labels showed at the genes track. Must be one of
#' "transcript" (default), "symbol", "gene", "exon" or "feature"
#' @param makePDF If TRUE, save the plot in pdf format in the working
#' directory. Otherwise, draw the plot in the active graphics window
#' @param col Vector with colors to plot methylation in different groups
#'
#' @return 'NULL'
#'
#' @author Jordi Martorell Marugán, \email{jordi.martorell@@genyo.es}
#'
#' @seealso \code{\link{rankProbes}}, \code{\link{mCSEATest}},
#' \code{\link{mCSEAPlotGSEA}}
#'
#' @examples
#' library(mCSEAdata)
#' data(mcseadata)
#' \dontrun{
#' myRank <- rankProbes(betaTest, phenoTest, refGroup = "Control")
#' set.seed(123)
#' myResults <- mCSEATest(myRank, betaTest, phenoTest,
#' regionsTypes = "promoters", platform = "EPIC")
#' }
#' data(precomputedmCSEA)
#' mCSEAPlot(myResults, "promoters", "CLIC6",
#' transcriptAnnotation = "symbol", makePDF = FALSE)
#' @export


mCSEAPlot <- function(mCSEAResults, regionType, dmrName,
                      extend = 1000,
                      chromosome = TRUE, leadingEdge = TRUE, CGI = FALSE,
                      genes = TRUE, transcriptAnnotation = "transcript",
                      makePDF = TRUE, 
                      col = c("blue", "magenta", "green", "red", "black")){
  
  progress <- utils::txtProgressBar(min=0, max=10, style=3)
  
  # Check input objects
  
  if (any(class(mCSEAResults) != "list" | length(mCSEAResults) < 5)){
    stop("mCSEAResults must be the object generated
                by mCSEATest function")
  }
  
  if (class(dmrName) != "character"){
    stop("dmrName must be a character object")
  }
  
  if (any(class(extend) != "numeric" | extend < 0)){
    stop("extend must be a positive number")
  }
  
  if (class(transcriptAnnotation) != "character"){
    stop("transcriptAnnotation must be a character object")
  }
  
  if (class(chromosome) != "logical"){
    stop("chromosome must be a logical object (TRUE/FALSE)")
  }
  
  if (class(leadingEdge) != "logical"){
    stop("leadingEdge must be a logical object (TRUE/FALSE)")
  }
  
  if (class(CGI) != "logical"){
    stop("CGI must be a logical object (TRUE/FALSE)")
  }
  
  if (class(genes) != "logical"){
    stop("genes must be a logical object (TRUE/FALSE)")
  }
  
  if (class(makePDF) != "logical"){
    stop("makePDF must be a logical object (TRUE/FALSE)")
  }
  if (!is.vector(col)){
    stop("col must be a character or numeric vector")
  }
  
  transcriptAnnotation <- match.arg(transcriptAnnotation,
                                    choices=c("transcript", "symbol",
                                              "gene", "exon", "feature"))
  
  pheno <- mCSEAResults[["pheno"]]
  methData <- mCSEAResults[["methData"]]
  platform <- mCSEAResults[["platform"]]
  
  genome <- ifelse(platform == "EPICv2", "hg38", "hg19") # new variable
  if (platform == "EPICv2") {
      bands <- mCSEAdata::bandTablehg38
  } else {
      bands <- mCSEAdata::bandTablehg19
  }  
  # Get the appropiate association
  
  regionType <- match.arg(regionType, choices=c("promoters", "genes", "CGI",
                                                "custom"))
  
  assoc <- mCSEAResults[[paste(regionType, "association", sep="_")]]
  
  leadingCpGs <- strsplit(mCSEAResults[[regionType]]
                          [dmrName, "leadingEdge"],", ")[[1]]
  
  cgs.plot <- assoc[[dmrName]]
  
  CpGs <- methData[cgs.plot,]
  
  utils::setTxtProgressBar(progress, 1)
  
  if (platform == "450k") {
    utils::setTxtProgressBar(progress, 2)
    annot <- mCSEAdata::annot450K
  } else if (platform == "EPIC") {
    utils::setTxtProgressBar(progress, 2)
    annot <- mCSEAdata::annotEPIC
  } else{ # set up
    utils::setTxtProgressBar(progress, 2)
    annot <- mCSEAdata::annotEPICv2
  }
  
  utils::setTxtProgressBar(progress, 3)
  
  positions <- IRanges::start(IRanges::ranges(annot[c(cgs.plot),]))
  Chromosome <- as.character(GenomicRanges::seqnames(annot[c(cgs.plot[1])]))
  From <- min(positions) - extend
  To <- max(positions) + extend
  
  #Check for annotation
  annotSubset <- annot[names(annot) %in% cgs.plot,]
  probes <- names(annotSubset)
  
  Phenotype <- factor(as.character(pheno[,1]))
  
  
  # Subset data
  dataSubset <- methData[na.omit(match(probes, rownames(methData))),]
  
  utils::setTxtProgressBar(progress, 4)
  
  if (chromosome) {
    #Ideogram
    itrack <- Gviz::IdeogramTrack(genome=genome, chromosome=Chromosome, #set up
                                  bands=bands)
    #Genome axis
    gtrack <- Gviz::GenomeAxisTrack() 
  }
  utils::setTxtProgressBar(progress, 5)
  
  
  if (genes){
    #Genes track from UCSC
    
    # set up accession to biomart
    if (platform == "EPICv2"){
      bm <- biomaRt::useMart(biomart="ENSEMBL_MART_ENSEMBL", 
                             dataset="hsapiens_gene_ensembl")
      
    } else{
      bm <- biomaRt::useMart(host="https://grch37.ensembl.org", # modified due to warning
                             biomart="ENSEMBL_MART_ENSEMBL", 
                             dataset="hsapiens_gene_ensembl")
    }
    biomTrack <- Gviz::BiomartGeneRegionTrack(genome=genome,  # set up
                                              chromosome=Chromosome,
                                              start=From, end=To ,
                                              name="ENSEMBL annotation",
                                              stacking="squish",
                                              just.group="above",
                                              transcriptAnnotation=
                                                transcriptAnnotation,
                                              biomart=bm)
    
    
    
  }
  utils::setTxtProgressBar(progress, 6)
  
  
  #CpG islands
  if(CGI) {
    cpgIslands <- Gviz::UcscTrack(genome=genome, chromosome=Chromosome, # set up
                                  track="cpgIslandExt", from=From,
                                  to=To,
                                  trackType="AnnotationTrack",
                                  start="chromStart", end="chromEnd",
                                  showId=FALSE, shape="box",
                                  fill="#006400", name="CpG Islands")
  }
  utils::setTxtProgressBar(progress, 7)
  
  #Data track
  annotSubset <- annotSubset[match(rownames(dataSubset),
                                   names(annotSubset)),]
  
  #Reorder columns
  dataSubsetOrdered <- dataSubset[,order(Phenotype)]
  phenotypeOrdered <- Phenotype[order(Phenotype)]
  
  nSamples <- nrow(dataSubsetOrdered)
  
  #Place data
  dataValues <- data.frame(c(data.frame(annotSubset)[,c("start","start",
                                                        "seqnames")]),
                           as.data.frame(dataSubsetOrdered))
  rownames(dataValues) <- rownames(dataSubsetOrdered)
  colnames(dataValues)[1] <- "start"
  colnames(dataValues)[2] <- "end"
  colnames(dataValues)[3] <- "chromosome"
  dataValues[2] <- dataValues[2] + 1
  
  if (length(col) < nlevels(phenotypeOrdered)){
    stop("You specified less colors than phenotype classes")
  }
  
  dtrack <- Gviz::DataTrack(dataValues, genome=genome, #set up
                            type=c("g","p","a"),
                            name="DNA Methylation",
                            groups=phenotypeOrdered,
                            col = col)
  
  utils::setTxtProgressBar(progress, 8)
  
  
  if (leadingEdge){
    
    leadingValues <- data.frame(c(dataValues[,seq_len(3)]), -1,
                                row.names=rownames(dataValues))
    
    for (site in rownames(leadingValues)) {
      if (site %in% leadingCpGs){
        leadingValues[site,4] <- 1
      }
    }
    
    if (sum(leadingValues[,4]) == nrow(leadingValues)) {
      dtrack2 <- Gviz::DataTrack(leadingValues, genome=genome, #set up
                                 type="heatmap",
                                 name="KS leading edge",
                                 ncolor=2, gradient=c("green", "red"),
                                 showAxis=FALSE)
    }
    
    else {
      dtrack2 <- Gviz::DataTrack(leadingValues, genome=genome, #set up
                                 type="heatmap",
                                 name="KS leading edge",
                                 ncolor=2, gradient=c("red", "green"),
                                 showAxis=FALSE)
    }
  }
  utils::setTxtProgressBar(progress, 9)
  
  
  
  #Scale PDF
  
  if (genes){
    lengthGenes <- length(unique(GenomicRanges::restrict(attr(
      biomTrack, "range"), From, To)$transcript))
    heightPdf <- sum(chromosome, CGI, leadingEdge) + 0.5*lengthGenes + 1
    heightGeneTrack <- 2 + 0.5*lengthGenes
  }
  
  else {
    heightPdf <- sum(chromosome, CGI, leadingEdge, genes) + 2
  }
  
  if (makePDF){
    pdf(paste0(dmrName,".pdf"), height=heightPdf)
  }
  
  # Plot tracks
  
  if (chromosome) {
    if (leadingEdge & CGI & genes){
      Gviz::plotTracks(list(itrack, gtrack, dtrack, dtrack2, cpgIslands,
                            biomTrack),
                       from=From, to=To,
                       background.panel="#FFFFFF",
                       background.title="darkblue", lwd=1,
                       legend=TRUE,
                       sizes=c(0.5,1,3,1,2,heightGeneTrack),
                       background.panel="white",
                       background.title="#2e2e2e")
    }
    else if (CGI & genes){
      Gviz::plotTracks(list(itrack, gtrack, dtrack,
                            cpgIslands, biomTrack),
                       from=From, to=To, background.panel="#FFFFFF",
                       background.title="darkblue", lwd=1,
                       legend=TRUE,
                       sizes=c(0.5,1,3,2,heightGeneTrack),
                       background.panel="white",
                       background.title="#2e2e2e")
    }
    else if (leadingEdge & genes){
      Gviz::plotTracks(list(itrack, gtrack, dtrack, dtrack2, biomTrack),
                       from=From, to=To, background.panel="#FFFFFF",
                       background.title="darkblue", lwd=1,
                       legend=TRUE,
                       sizes=c(0.5,1,3,1,heightGeneTrack),
                       background.panel="white",
                       background.title="#2e2e2e")
    }
    else if (leadingEdge & CGI){
      Gviz::plotTracks(list(itrack, gtrack, dtrack, dtrack2, cpgIslands),
                       from=From, to=To, background.panel="#FFFFFF",
                       background.title="darkblue", lwd=1,
                       legend=TRUE,
                       sizes=c(0.5,1,3,1,2), background.panel="white",
                       background.title="#2e2e2e")
    }
    else if (leadingEdge){
      Gviz::plotTracks(list(itrack, gtrack, dtrack, dtrack2), from=From,
                       to=To, background.panel="#FFFFFF",
                       background.title="darkblue", lwd=1,
                       legend=TRUE,
                       sizes=c(0.5,1,3,1), background.panel="white",
                       background.title="#2e2e2e")
    }
    else if (CGI){
      Gviz::plotTracks(list(itrack, gtrack, dtrack, cpgIslands),
                       from=From,
                       to=To, background.panel="#FFFFFF",
                       background.title="darkblue", lwd=1,
                       legend=TRUE,
                       sizes=c(0.5,1,3,2), background.panel="white",
                       background.title="#2e2e2e")
    }
    else if (genes){
      Gviz::plotTracks(list(itrack, gtrack, dtrack, biomTrack),
                       from=From,
                       to=To, background.panel="#FFFFFF",
                       background.title="darkblue", lwd=1,
                       legend=TRUE,
                       sizes=c(0.5,1,3,heightGeneTrack),
                       background.panel="white",
                       background.title="#2e2e2e")
    }
    else {
      Gviz::plotTracks(list(itrack, gtrack, dtrack), from=From, to=To,
                       background.panel="#FFFFFF",
                       background.title="darkblue", lwd=1,
                       legend=TRUE,
                       sizes=c(0.5,1,3), background.panel="white",
                       background.title="#2e2e2e")
    }
  }
  
  else if (leadingEdge) {
    if (CGI & genes){
      Gviz::plotTracks(list(dtrack, dtrack2, cpgIslands, biomTrack),
                       from=From, to=To, background.panel="#FFFFFF",
                       background.title="darkblue", lwd=1,
                       legend=TRUE,
                       sizes=c(3,1,2,heightGeneTrack),
                       background.panel="white",
                       background.title="#2e2e2e")
    }
    else if (genes){
      Gviz::plotTracks(list(dtrack, dtrack2, biomTrack), from=From,
                       to=To,
                       background.panel="#FFFFFF",
                       background.title="darkblue", lwd=1,
                       legend=TRUE,
                       sizes=c(3,1,heightGeneTrack),
                       background.panel="white",
                       background.title="#2e2e2e")
    }
    else if (CGI){
      Gviz::plotTracks(list(dtrack, dtrack2, cpgIslands), from=From,
                       to=To,
                       background.panel="#FFFFFF",
                       background.title="darkblue", lwd=1,
                       legend=TRUE,
                       sizes=c(3,1,2), background.panel="white",
                       background.title="#2e2e2e")
    }
    else {
      Gviz::plotTracks(list(dtrack, dtrack2), from=From, to=To,
                       background.panel="#FFFFFF",
                       background.title="darkblue", lwd=1,
                       legend=TRUE,
                       sizes=c(3,1), background.panel="white",
                       background.title="#2e2e2e")
    }
  }
  
  else if (CGI) {
    if (genes){
      Gviz::plotTracks(list(dtrack, cpgIslands, biomTrack), from=From,
                       to=To, background.panel="#FFFFFF",
                       background.title="darkblue", lwd=1,
                       legend=TRUE,
                       sizes=c(3,2,heightGeneTrack),
                       background.panel="white",
                       background.title="#2e2e2e")
    }
    else {
      Gviz::plotTracks(list(dtrack, cpgIslands), from=From, to=To,
                       background.panel="#FFFFFF",
                       background.title="darkblue", lwd=1,
                       legend=TRUE,
                       sizes=c(3,2), background.panel="white",
                       background.title="#2e2e2e")
    }
  }
  
  else if (genes) {
    Gviz::plotTracks(list(dtrack, biomTrack), from=From, to=To,
                     background.panel="#FFFFFF",
                     background.title="darkblue", lwd=1, legend=TRUE,
                     sizes=c(3,heightGeneTrack),
                     background.panel="white",
                     background.title="#2e2e2e")
  }
  
  else {
    Gviz::plotTracks(list(dtrack), from=From, to=To,
                     background.panel="#FFFFFF",
                     background.title="darkblue", lwd=1, legend=TRUE,
                     sizes=c(3), background.panel="white",
                     background.title="#2e2e2e")
  }
  
  if (makePDF){
    dev.off()
  }
  
  utils::setTxtProgressBar(progress, 10)
  
}

#' Integrate methylation and expression
#'
#' Uses mCSEA methylation analysis results and expression values to search for
#' significant correlations between DMRs methylation and close genes expression.
#'
#' @param mCSEAResults The object generated by mCSEATest function
#' @param exprData A matrix or data frame with genes in rows and samples in
#' columns. A SummarizedExperiment object can be used too
#' @param regionType The region types to be represented. Must be one or more of
#' "promoters", "genes", "CGI" and "custom"
#' @param geneIDs Gene identifiers used in exprData. One of "SYMBOL",
#' "ENSEMBL", "ENTREZID", "GENEID", "REFSEQ" or "UNIGENE"
#' @param dmrName The DMR of interest to correlate with expression (e.g. gene
#' name, CGI name...). If NULL (default), all DMRs with P-Value < pcutoff are
#' selected
#' @param pcutoff P-Value threshold to select DMRs if dmrName = NULL
#' @param minCor Correlation threshold to output the results
#' @param minP Correlation P-Value threshold to output the results
#' @param makePlot If TRUE, generate corelation and save them in the folder
#' specified by folder parameter
#' @param folder Directory to save the correlation plots if makePlot = TRUE
#' @param nproc Number of processors to be used
#'
#' @return A data.frame with the integration results.
#'
#' @author Jordi Martorell Marugán, \email{jordi.martorell@@genyo.es}
#'
#' @seealso \code{\link{rankProbes}}, \code{\link{mCSEATest}}
#'
#' @examples
#' data(precomputedmCSEA)
#' data(exprTest)
#'
#' resultsInt <- mCSEAIntegrate(myResults, exprTest, "promoters", "ENSEMBL",
#'                             "GATA2", makePlot = FALSE)
#'
#' resultsInt
#'
#' @export

mCSEAIntegrate <- function(mCSEAResults, exprData,
                            regionType = c("promoters", "genes", "CGI",
                                            "custom"),
                            geneIDs = "SYMBOL",
                            dmrName = NULL,
                            pcutoff = 0.05,
                            minCor = 0.5, minP = 0.05, makePlot = TRUE,
                            folder = ".", nproc = 1){

    regionType <- match.arg(regionType, choices=c("promoters", "genes", "CGI",
                                                    "custom"), several.ok=TRUE)

    geneIDs <- match.arg(geneIDs, choices=c("SYMBOL", "ENSEMBL",
                                            "ENTREZID", "GENEID",
                                            "REFSEQ", "UNIGENE"))

    pheno <- mCSEAResults[["pheno"]]
    methData <- mCSEAResults[["methData"]]
    platform <- mCSEAResults[["platform"]]

    # Remove genes with Standar Deviation = 0
    ngenesPre <- nrow(exprData)
    exprData <- exprData[apply(exprData, 1, sd) != 0,]
    ngenesPost <- nrow(exprData)
    
    message(ngenesPre -  ngenesPost, " genes removed from exprData due to Standar Deviation = 0")

    # Check input objects

    if (any(class(mCSEAResults) != "list" | length(mCSEAResults) < 5)){
        stop("mCSEAResults must be the object generated
            by mCSEATest function")
    }

    if (!any(class(exprData) == "data.frame" | class(exprData) == "matrix" |
            class(exprData) == "SummarizedExperiment" |
            class(exprData) == "RangedSummarizedExperiment")){
        stop("exprData must be a data frame, a matrix or a SummarizedExperiment
            object")
    }

    if (class(geneIDs) != "character"){
        stop("geneIDs must be a character object")
    }

    if (class(dmrName) != "character" & !is.null(dmrName)){
        stop("dmrName must be a character object or NULL")
    }

    if (any(class(pcutoff) != "numeric" | pcutoff < 0)){
        stop("pcutoff must be a positive number")
    }

    if (any(class(minCor) != "numeric" | minCor < 0)){
        stop("minCor must be a positive number")
    }

    if (any(class(minP) != "numeric" | minP < 0)){
        stop("minP must be a positive number")
    }

    if (class(makePlot) != "logical"){
        stop("makePlot must be a logical object (TRUE7FALSE)")
    }

    if (class(folder) != "character"){
        stop("folder must be a character object")
    }

    if (!file.exists(folder)){
        dir.create(folder)
    }

    if (any(class(nproc) != "numeric" | nproc < 0)){
        stop("nproc must be a positive number")
    }

    if ("SummarizedExperiment" %in% is(exprData) |
        "RangedSummarizedExperiment" %in% is(exprData)){
        exprData <- SummarizedExperiment::assay(exprData)
    }

    if (!identical(colnames(methData), colnames(exprData))) {
        if (length(setdiff(colnames(methData), colnames(exprData))) == 0 &&
            length(setdiff(colnames(exprData), colnames(methData))) == 0) {
            exprData <- exprData[,colnames(methData)]
        }

        else {
            stop("Sample labels of methData and exprData must be the same")
        }
    }


    if (length(levels(pheno[,1])) != 2){
        stop("Integration can only be performed with a 2 groups comparison")
    }

    for (reg in regionType) {
        if (is.null(mCSEAResults[[reg]])){
            warning("mCSEAResults does not contain ", reg, " regionType. ",
                    "Skipping it.")
            regionType = regionType[!regionType %in% reg]
        }
    }

    if (length(regionType) == 0) {
        stop("None of the specified regionTypes in mCSEAResults")
    }


    intResults <- data.frame()

    exprData <- as.data.frame(exprData)

    rangeGenes <- suppressMessages(GenomicFeatures::genes(
        Homo.sapiens::Homo.sapiens, columns = c("CDSCHROM", geneIDs)))

    for (reg in regionType) {

        message("Integrating ", reg, " methylation with gene expression")

        sigRegions <- mCSEAResults[[reg]]
        sigRegions <- sigRegions[sigRegions[, "pval"] < pcutoff,]

        if (is.null(dmrName)) {
            region2 <- rownames(sigRegions)
        }
        else {
            region2 <- dmrName
        }

        corCpGs <- strsplit(sigRegions[region2,"leadingEdge"], ", ")
        names(corCpGs) <- region2

        if (platform == "450k") {
            annot <- mCSEAdata::annot450K
        }
        else {
            annot <- mCSEAdata::annotEPIC
        }

        cl <- parallel::makeCluster(nproc)

        parallel::clusterExport(cl=cl,
                                varlist=c("region2", "methData", "exprData",
                                            "pheno", "geneIDs",
                                            "minCor", "minP", "reg", "corCpGs",
                                            "annot", "rangeGenes", "folder",
                                            "makePlot", ".integrateCore"),
                                envir=environment())

        indInt <- parallel::parLapply(cl, region2,
                            fun = function(x) {
                                .integrateCore(x, methData=methData,
                                                exprData=exprData, pheno=pheno,
                                                geneIDs=geneIDs,
                                                minCor=minCor, minP=minP,
                                                regionType=reg, corCpGs=corCpGs,
                                                annot=annot,
                                                rangeGenes=rangeGenes,
                                                folder=folder,
                                                makePlot=makePlot)})
        


        parallel::stopCluster(cl)
        
		intResults <- rbind(intResults, do.call("rbind", indInt))
    }

    intResults <- intResults[order(abs(intResults[["Correlation"]]),
                                    decreasing=TRUE),]
    intResults[["adjPValue"]] <- p.adjust(intResults[["PValue"]],
                                            method = "fdr")
    rownames(intResults) <- seq_len(nrow(intResults))

    return(intResults)

}

.integrateCore <- function(indRegion, methData, exprData, pheno, geneIDs,
                            minCor, minP, regionType, corCpGs, annot,
                            rangeGenes, folder, makePlot){
	
    corCpGs <- corCpGs[[indRegion]]

    pheno <- pheno[,1]

    if (length(corCpGs) > 1) {
        meth1 <- colMeans(methData[corCpGs, pheno == levels(pheno)[1]], 
                            na.rm = TRUE)
        meth2 <- colMeans(methData[corCpGs, pheno == levels(pheno)[2]], 
                            na.rm = TRUE)
	}
	else {
		meth1 <- unlist(methData[corCpGs, pheno == levels(pheno)[1]])
		meth2 <- unlist(methData[corCpGs, pheno == levels(pheno)[2]])
	}
    meth <- c(meth1, meth2)


    annotCpGs <- annot[corCpGs]

    nearGenes <- IRanges::subsetByOverlaps(rangeGenes, annotCpGs, maxgap = 1500)

    for (gene in unlist(S4Vectors::mcols(nearGenes)[,geneIDs])){

        if (gene %in% rownames(exprData)){

            feat <- gen <- correlations <- pvals <- c()

            expr1 <- colMeans(exprData[gene, pheno == levels(pheno)[1]],
                                na.rm = TRUE)
            expr2 <- colMeans(exprData[gene, pheno == levels(pheno)[2]],
                                na.rm = TRUE)
            expr <- c(expr1, expr2)

            corResult <- try(cor.test(meth, expr))

            interesting <- TRUE

            if (class(corResult) == "try-error") {
            	interesting <- FALSE
            }

            else if (regionType == "promoters" & corResult[["estimate"]] > 0) {
                interesting <- FALSE
            }

            else if (regionType == "genes" & corResult[["estimate"]] < 0) {
                interesting <- FALSE
            }

            if (corResult[["p.value"]] < minP &
                abs(corResult[["estimate"]]) > minCor &
                interesting){


                feat <- c(feat, indRegion)
                gen <- c(gen, gene)
                correlations <- c(correlations, corResult[["estimate"]])
                pvals <- c(pvals, corResult[["p.value"]])

                xLim <- c(min(expr1, expr2), max(expr1, expr2))
                yLim <- c(min(meth1, meth2), max(meth1, meth2))

                if (makePlot){

                    if (regionType == "promoters"){
                        mainTitle <- paste0(indRegion,
                                            " promoter methylation vs. ",
                                            gene, " expression\nCorrelation = ",
                                            round(corResult[["estimate"]], 2),
                                            "   P-value = ",
                                            signif(corResult[["p.value"]], 2))
                    }
                    else if (regionType == "genes") {
                        mainTitle <- paste0(indRegion, " body methylation vs. ",
                                            gene, " expression\nCorrelation = ",
                                            round(corResult[["estimate"]], 2),
                                            "   P-value = ",
                                            signif(corResult[["p.value"]], 2))
                    }

                    else if (regionType == "CGI"){
                        mainTitle <- paste0(indRegion, " CGI methylation vs. ",
                                            gene, " expression\nCorrelation = ",
                                            round(corResult[["estimate"]], 2),
                                            "   P-value = ",
                                            signif(corResult[["p.value"]], 2))
                    }

                    else {
                        mainTitle <- paste0(indRegion, " methylation vs. ",
                                            gene, " expression\nCorrelation = ",
                                            round(corResult[["estimate"]], 2),
                                            "   P-value = ",
                                            signif(corResult[["p.value"]], 2))
                    }

                    pdf(paste0(folder, "/", indRegion, "_", gene, "_",
                                regionType, ".pdf"))

                    plot(expr1, meth1, xlim=xLim, ylim=yLim, main=mainTitle,
                        xlab="Expression", ylab="Methylation", pch=16,
                        col="darkred")
                    par(new=TRUE)
                    plot(expr2, meth2, xlim=xLim, ylim=yLim, xaxt="n",
                        yaxt="n", xlab="", ylab="", pch=16, col="dodgerblue3")

                    if (corResult[["estimate"]] < 0){

                        legend("topright", legend=levels(pheno), pch=16,
                                col=c("darkred", "dodgerblue3"))
                    }

                    else {
                        legend("bottomright", legend=levels(pheno), pch=16,
                                col=c("darkred", "dodgerblue3"))
                    }

                    abline(lm(meth ~ expr), lty=2)

                    dev.off()
                }
            }
        }
    }

    if (exists("feat") && length(feat) > 0) {
        out <- data.frame(Feature=feat,
                            regionType=regionType,
                            Gene=gen, Correlation=correlations,
                            PValue=pvals)
        return(out)
    }
}

test_mCSEAIntegrate <- function() {
    library(leukemiasEset)
    data(exprTest)
    resultsInt <- mCSEAIntegrate(myResults, exprTest, "promoters", "ENSEMBL", "GATA2",
                                    makePlot = FALSE)
    checkEqualsNumeric(resultsInt[1,4],
                    -0.8908771, tolerance=1.0e-6)
}

---
title: "DMRs identification with mCSEA package"
author:
- name: Jordi Martorell-Marugán
  affiliation:
  - Bioinformatics Unit. GENYO, Centre for Genomics and Oncological Research
- name: Pedro Carmona-Sáez
  affiliation:
  - Bioinformatics Unit. GENYO, Centre for Genomics and Oncological Research
  email: pedro.carmona@genyo.es
package: mCSEA
date: "`r BiocStyle::doc_date()`"
abstract: >
  mCSEA (methylathed CpGs Set Enrichment Analysis) searches Differentially
  Methylated Regions (DMRs) between conditions using methylation data from 
  Illumina's 450k or EPIC microarrays. The evaluated DMRs are predefined 
  regions (promoters, gene bodies, CpG Islands and user-defined regions).
  This package contains functions to rank the CpG probes, to apply a 
  GSEA analysis for DMRs identification, to plot the results and to integrate 
  them with expression data.
output:
  # BiocStyle::pdf_document(template = NULL)
  pdf_document:
    template: NULL
#output: word_document

vignette: >
  %\VignetteIndexEntry{Predefined DMRs identification with mCSEA package}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
references:
- id: du10
  title: "Comparison of Beta-value and M-value methods for quantifying methylation levels by microarray analysis"
  author:
  - family: Du
    given: P
  - family: Zhang
    given: X
  - family: Huang
    given: C
  - family: Jafari
    given: N
  - family: Kibbe
    given: WA
  - family: Hou
    given: L
  - family: Lin
    given: SM
  container-title: BMC Bioinformatics
  issued:
    year: 2010
  volume: 11
  issue: 587
- id: jones12
  title: "Functions of DNA Methylation: islands, start sites, gene bodies and beyond"
  author:
  - family: Jones
    given: Peter A.
  container-title: Nature Reviews Genetics
  issued:
    year: 2012
  volume: 13
  issue: 7
  
  
- id: mcgregor16
  title: "An evaluation of methods correcting for cell-type heterogeneity in DNA methylation studies"
  author:
  - family: McGregor
    given: K
  - family: Bernatsky
    given: S
  - family: Colmegna
    given: I
  - family: Pastinen
    given: T
  - family: Labbe
    given: A
  - family: Greenwood
    given: CMT
  container-title: Genome Biology
  issued:
    year: 2016
  volume: 17
  issue: 84
  
- id: jones12
  title: "Functions of DNA Methylation: islands, start sites, gene bodies and beyond"
  author:
  - family: Jones
    given: Peter A.
  container-title: Nature Reviews Genetics
  issued:
    year: 2012
  volume: 13
  issue: 7
---

# Previous steps

The input of 
mCSEA is tipically a matrix with the processed $\beta$-values for each probe 
and sample. If you start from the raw methylation files (like .idat files) you 
should first preprocess the data with any of the available packages for 
that purpose (e.g. `r BiocStyle::Biocpkg("minfi")` or 
`r BiocStyle::Biocpkg("ChAMP")`). Minfi 
includes functions to get a matrix of $\beta$-values (_getBeta()_) or 
M-values (_getM()_). ChAMP output class depends on the type of analysis 
performed. For instance, _champ.norm()_ function returns a matrix, while 
_champ.load()_ returns a list of results, and one of them is a $\beta$-values
matrix. So mCSEA is totally compatible with minfi and ChAMP outputs as long as 
a matrix with the methylation values is obtained.

## Reading .idat files

Here we provide a minimal example to read .idat files with 
`r BiocStyle::Biocpkg("minfi")` package. A samplesheet must 
be provided in order to read the raw files and generate a 
_RGChannelSet_ object. For more information about this step, 
read minfi's [vignette](https://bioconductor.org/packages/release/bioc/vignettes/minfi/inst/doc/minfi.html#3_reading_data).

```{r, message = FALSE}
library(minfi)
minfiDataDir <- system.file("extdata", package = "minfiData")
targets <- read.metharray.sheet(minfiDataDir, verbose = FALSE)
RGset <- read.metharray.exp(targets = targets)
```


## Cell type heterogeneity correction

Different cell types proportions across samples are one of the major 
sources of variability in methylation data from tissues like blood or 
saliva (@mcgregor16). There are a lot of packages which can be used to 
estimate cell types proportions in each sample in order to correct for 
this bias (reviewed in @mcgregor16). Here we supply an example where blood 
reference data is used to estimate cell types proportions of each blood sample.

```{r, message=FALSE}
library(FlowSorted.Blood.450k)
library(mCSEA)
data(mcseadata)

cellCounts = estimateCellCounts(RGset)
print(cellCounts)
```

These proportions could be introduced as covariates in the linear models.

# Step 1: Ranking CpGs probes

To run a mCSEA analysis, you must rank all the evaluated CpGs probes with some 
metric (e.g. t-statistic, Fold-Change...). You can use _rankProbes()_ function 
for that aim, or prepare a ranked list with the same structure as the 
_rankProbes()_ output.

We load sample data to show how _rankProbes()_ works: 

```{r, message = FALSE, results='hide'}
library(mCSEA)
data(mcseadata)
```

We loaded to our R environment **betaTest** and **phenoTest** objects, in 
addition to **exprTest**, annotation objects and association objects 
(we will talk about these after). **betaTest** 
is a matrix with the $\beta$-values of 10000 EPIC probes for 20 samples. 
**phenoTest** is a dataframe with the explanatory variable and covariates 
associated to the samples. When you load your own data, the structure of 
your objects should be similar.

```{r}
head(betaTest, 3)
print(phenoTest)
```

_rankProbes()_ function uses these two objects as input and apply a linear 
model with `r BiocStyle::Biocpkg("limma")` package. By default, _rankProbes()_ 
considers the first column of the phenotypes table as the explanatory variable 
in the model (e.g. cases and controls) and does not take into account any 
covariate to adjust the models. You can change this behaviour modifying 
**explanatory** and **covariates** options.

By default, _rankProbes()_ assumes that the methylation data object contains 
$\beta$-values and transform them to M-values before calculating the linear 
models. If your methylation data object contains M-values, you must specify 
it (typeInput = "M"). You can also use $\beta$-values for models calculation 
(typeAnalysis = "beta"), although we do not recommend it due to it has been 
proven that M-values better accomplish the statistical assumptions of limma 
analysis (@du10).

```{r}
myRank <- rankProbes(betaTest, phenoTest, refGroup = "Control")
```
**myRank** is a named vector with the t-values for each CpG probe.

```{r}
head(myRank)
```

You can also supply _rankProbes()_ function with a SummarizedExperiment 
object. In that case, if you don't specify a **pheno** object, phenotypes 
will be extracted from the SummarizedExperiment object with _colData()_ 
function.

## Paired analysis

It is possible to take into account paired samples in _rankProbes()_
analysis. For that aim, you should use paired = TRUE parameter and 
specify the column in pheno containing pairing information 
(pairColumn parameter).

# Step 2: Searching DMRs in predefined regions

Once you calculated a score for each CpG, you can perform the mCSEA analysis. 
For that purpose, you should use _mCSEATest()_ function. This function takes 
as input the vector generated in the previous step, the methylation data and 
the phenotype information. By default, it searches 
for differentially methylated promoters, gene bodies and CpG Islands. You can 
specify the regions you want to test with _regionsTypes_ option. _minCpGs_ 
option specifies the minimum amount of CpGs in a region to be considered in 
the analysis (5 by default). You can increase the number of processors to use 
with _nproc_ option (recommended if you have enough computational resources). 
Finally, you should specify if the array platform is 450k or EPIC with 
the _platform_ option. 

```{r, warning=FALSE}
myResults <- mCSEATest(myRank, betaTest, phenoTest, 
                        regionsTypes = "promoters", platform = "EPIC")
```

_mCSEATest()_ returns a list with the GSEA results and the association objects 
for each region type analyzed, in addition to the input data (methylation, 
phenotype and platform).

```{r}
ls(myResults)
```

**promoters** is a data frame with the following columns (partially extracted 
from `r BiocStyle::Biocpkg("fgsea")` help):

* *pval:* Estimated P-value.
* *padj:* P-value adjusted by BH method.
* *log2err:* Expected error for the standard deviation of the P-value logarithm.
* *ES:* Enrichment score.
* *NES:* Normalized enrichment score by number of CpGs associated to the 
feature.
* *size:* Number of CpGs associated to the feature.
* *leadingEdge:* Leading edge CpGs which drive the enrichment.

```{r}
head(myResults[["promoters"]][,-7])
```

On the other hand, **promoters_association** is a list with the CpG probes 
associated to each feature:

```{r}
head(myResults[["promoters_association"]], 3)
```

You can also provide a custom association object between CpG 
probes and regions (_customAnnotation_ option). This object should be a list 
with a structure similar to this:

```{r}
head(assocGenes450k, 3)
```

# Step 3: Plotting the results

Once you found some DMRs, you can make a plot with the genomic context of the 
interesting ones. For that, you must provide _mCSEAPlot()_ function with the 
_mCSEATest()_ results, and 
you must specify which type of region you want to plot and the name of the 
DMR to be plotted (e.g. gene name). There are some graphical parameters you can
adjust (see _mCSEAPlot()_ help). Take into account that this function connects 
to some online servers in order to get genomic information. For that reason, 
this function could take some minutes to finish the plot, specially the first 
time it is executed.

```{r, message = FALSE, results='hide'}
mCSEAPlot(myResults, regionType = "promoters", 
           dmrName = "CLIC6",
           transcriptAnnotation = "symbol", makePDF = FALSE)
```

You can also plot the GSEA results for a DMR with _mCSEAPlotGSEA()_ 
function.

```{r}
mCSEAPlotGSEA(myRank, myResults, regionType = "promoters", dmrName = "CLIC6")
```

# Integrating methylation and expression data

If you have both methylation and expression data for the same samples, you can 
integrate them in order to discover significant associations between 
methylation changes in a DMR and an expression alterations in a close gene.
_mCSEAIntegrate()_ considers the DMRs identified by _mCSEATest()_ passing a 
P-value threshold (0.05 by default). It calculates the mean methylation for 
each condition using the leading edge CpGs and performs a correlation test 
between this mean DMR methylation and the expression of close genes. This 
function automatically finds the genes located within a determined distance 
(1.5 kb) from the DMR. Only correlations passing thresholds (0.5 for 
correlation value and 0.05 por P-value by default) are returned. For 
promoters, only negative correlations are returned due to this kind of 
relationship between promoters methylation and gene expression has been 
largely observed (@jones12). On the contrary, only positive correlations 
between gene bodies methylation and gene expression are returned, due to this 
is a common relationship observed (@jones12). For CpG islands and custom 
regions, both positive and negative correlations are returned, due to they can 
be located in both promoters and gene bodies.

To test this function, we extracted a subset of 100 genes expression from bone 
marrows of 10 healthy and 10 leukemia patients (**exprTest**). Data was 
extracted from `r BiocStyle::Biocpkg("leukemiasEset")` package.

```{r}
# Explore expression data
head(exprTest, 3)

# Run mCSEAIntegrate function
resultsInt <- mCSEAIntegrate(myResults, exprTest, "promoters", "ENSEMBL")

resultsInt
```

It is very important to specify the correct gene identifiers used in the 
expression data (_geneIDs_ parameter). _mCSEAIntegrate()_ automatically 
generates correlation plots for the significant results and save them in the 
directory specified by _folder_ parameter (current directory by default).

![Integration plot for GATA2 promoter methylation and ENSG00000179348 
expression. Note that, actually, both names refers to the same gene, but 
SYMBOL was used to analyze promoters methylation and ENSEMBL ID was used as 
gene identifiers in the expression data.](GATA2_ENSG00000179348_promoters.pdf)

# Session info
```{r sessionInfo, echo=FALSE}
sessionInfo()
```

# References

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rankProbes.R
\name{rankProbes}
\alias{rankProbes}
\title{Rank CpG probes}
\usage{
rankProbes(methData, pheno = NULL, paired = FALSE, explanatory = 1,
  covariates = c(), pairColumn = c(), caseGroup = 1, refGroup = 2,
  continuous = NULL, typeInput = "beta", typeAnalysis = "M")
}
\arguments{
\item{methData}{A data frame or a matrix containing Illumina's CpG probes in
rows and samples in columns. A SummarizedExperiment object can be used too}

\item{pheno}{A data frame or a matrix containing samples in rows and
covariates in columns. If NULL (default), pheno is extracted from the
SummarizedExperiment object}

\item{paired}{Perform a paired t-test (default = FALSE)}

\item{explanatory}{The column name or position from pheno used to perform the
comparison between groups (default = first column)}

\item{covariates}{A list or character vector with column names from pheno
used as data covariates in the linear model}

\item{pairColumn}{Only for paired analysis. The column name or position from
pheno used to connect the paired samples (default = NULL)}

\item{caseGroup}{The group name or position from explanatory variable used as
cases to perform the comparison (default = first group)}

\item{refGroup}{The group name or position from explanatory variable used as
reference to perform the comparison (default = second group)}

\item{continuous}{A list or character vector with columns names from pheno
which should be treated as continuous variables (default = none)}

\item{typeInput}{Type of input methylation data. "beta" for Beta-values and
"M" for M-values}

\item{typeAnalysis}{"M" to use M-values to rank the CpG probes (default).
"beta" to use Beta-values instead}
}
\value{
A named vector containing the t-values from the linear model for each
 CpG probe
}
\description{
Apply a linear model to Illumina's 450k or EPIC methylation data to get the
t-value of each CpG probe
}
\examples{
data(mcseadata)
myRank <- rankProbes(betaTest, phenoTest, refGroup = "Control")
head(myRank)
}
\references{
Smyth, G. K. (2005). \emph{Limma: linear models for microarray
data}. Bioinformatics and Computational Biology Solutions using R and
Bioconductor, 397-420.
}
\seealso{
\code{\link{mCSEATest}}
}
\author{
Jordi Martorell Marugán, \email{jordi.martorell@genyo.es}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mCSEAPlotGSEA.R
\name{mCSEAPlotGSEA}
\alias{mCSEAPlotGSEA}
\title{Plot mCSEA results}
\usage{
mCSEAPlotGSEA(rank, mCSEAResults, regionType, dmrName)
}
\arguments{
\item{rank}{A named numeric vector with the ranking statistic of each CpG
site}

\item{mCSEAResults}{The object generated by mCSEATest function}

\item{regionType}{The region type to be represented. Must be one of
"promoters", "genes", "CGI" or "custom"}

\item{dmrName}{The DMR of interest to be represented (e.g. gene name,
CGI name...)}
}
\value{
'NULL'
}
\description{
Generate an enrichment plot
}
\examples{
\dontrun{
library(mCSEAdata)
data(mcseadata)
myRank <- rankProbes(betaTest, phenoTest, refGroup = "Control")
set.seed(123)
myResults <- mCSEATest(myRank, regionsTypes = "promoters",
platform = "EPIC")
}
data(precomputedmCSEA)
mCSEAPlotGSEA(myRank, myResults, "promoters", "CLIC6")
}
\references{
Subramanian, A. et al (2005). \emph{Gene set enrichment analysis:
 A knowledge-based approach for interpreting genome-wide expression profiles}
 . PNAS 102, 15545-15550.
}
\seealso{
\code{\link{rankProbes}}, \code{\link{mCSEATest}},
\code{\link{mCSEAPlot}}
}
\author{
Jordi Martorell Marugán, \email{jordi.martorell@genyo.es}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mCSEAPlot.R
\name{mCSEAPlot}
\alias{mCSEAPlot}
\title{Plot mCSEA results}
\usage{
mCSEAPlot(mCSEAResults, regionType, dmrName, extend = 1000,
  chromosome = TRUE, leadingEdge = TRUE, CGI = FALSE, genes = TRUE,
  transcriptAnnotation = "transcript", makePDF = TRUE,
  col = c("blue", "magenta", "green", "red", "black"))
}
\arguments{
\item{mCSEAResults}{The object generated by mCSEATest function}

\item{regionType}{The region type to be represented. Must be one of
"promoters", "genes", "CGI" or "custom"}

\item{dmrName}{The DMR of interest to be represented (e.g. gene name, CGI
name...)}

\item{extend}{The number of base pairs to extend the plot around the DMR
of interest (default = 1000 bp)}

\item{chromosome}{If TRUE, represent the chromosome and genome axis}

\item{leadingEdge}{If TRUE, represent the bars indicating if each CpG belongs
to the mCSEA leading edge (green) or not (red)}

\item{CGI}{If TRUE, represent the annotated CpG islands}

\item{genes}{If TRUE, represent the annotated genes}

\item{transcriptAnnotation}{Labels showed at the genes track. Must be one of
"transcript" (default), "symbol", "gene", "exon" or "feature"}

\item{makePDF}{If TRUE, save the plot in pdf format in the working
directory. Otherwise, draw the plot in the active graphics window}

\item{col}{Vector with colors to plot methylation in different groups}
}
\value{
'NULL'
}
\description{
Generate a graphic with the genomic context of the selected DMR, showing
methylation status at each CpG site of different samples groups
}
\examples{
library(mCSEAdata)
data(mcseadata)
\dontrun{
myRank <- rankProbes(betaTest, phenoTest, refGroup = "Control")
set.seed(123)
myResults <- mCSEATest(myRank, betaTest, phenoTest,
regionsTypes = "promoters", platform = "EPIC")
}
data(precomputedmCSEA)
mCSEAPlot(myResults, "promoters", "CLIC6",
transcriptAnnotation = "symbol", makePDF = FALSE)
}
\seealso{
\code{\link{rankProbes}}, \code{\link{mCSEATest}},
\code{\link{mCSEAPlotGSEA}}
}
\author{
Jordi Martorell Marugán, \email{jordi.martorell@genyo.es}
}

\name{precomputedmCSEA}
\alias{precomputedmCSEA}
\alias{myRank}
\alias{myResults}
\docType{data}
\title{Precomputed mCSEA results}
\description{
    myRank is a result of rankProbes function.
    myResults is a result of mCSEATest function.
}
\usage{data(precomputedmCSEA)}
\format{
    vector (\code{myRank}) and list data.frame (\code{myResults})
}
\source{
    Both objects were obtained with the example commands in the mCSEA
    vignette.
} 
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mCSEAIntegrate.R
\name{mCSEAIntegrate}
\alias{mCSEAIntegrate}
\title{Integrate methylation and expression}
\usage{
mCSEAIntegrate(mCSEAResults, exprData, regionType = c("promoters", "genes",
  "CGI", "custom"), geneIDs = "SYMBOL", dmrName = NULL, pcutoff = 0.05,
  minCor = 0.5, minP = 0.05, makePlot = TRUE, folder = ".", nproc = 1)
}
\arguments{
\item{mCSEAResults}{The object generated by mCSEATest function}

\item{exprData}{A matrix or data frame with genes in rows and samples in
columns. A SummarizedExperiment object can be used too}

\item{regionType}{The region types to be represented. Must be one or more of
"promoters", "genes", "CGI" and "custom"}

\item{geneIDs}{Gene identifiers used in exprData. One of "SYMBOL",
"ENSEMBL", "ENTREZID", "GENEID", "REFSEQ" or "UNIGENE"}

\item{dmrName}{The DMR of interest to correlate with expression (e.g. gene
name, CGI name...). If NULL (default), all DMRs with P-Value < pcutoff are
selected}

\item{pcutoff}{P-Value threshold to select DMRs if dmrName = NULL}

\item{minCor}{Correlation threshold to output the results}

\item{minP}{Correlation P-Value threshold to output the results}

\item{makePlot}{If TRUE, generate corelation and save them in the folder
specified by folder parameter}

\item{folder}{Directory to save the correlation plots if makePlot = TRUE}

\item{nproc}{Number of processors to be used}
}
\value{
A data.frame with the integration results.
}
\description{
Uses mCSEA methylation analysis results and expression values to search for
significant correlations between DMRs methylation and close genes expression.
}
\examples{
data(precomputedmCSEA)
data(exprTest)

resultsInt <- mCSEAIntegrate(myResults, exprTest, "promoters", "ENSEMBL",
                            "GATA2", makePlot = FALSE)

resultsInt

}
\seealso{
\code{\link{rankProbes}}, \code{\link{mCSEATest}}
}
\author{
Jordi Martorell Marugán, \email{jordi.martorell@genyo.es}
}

\name{exprTest}
\alias{exprTest}
\docType{data}
\title{Expression data example}
\description{
    exprTest is a subset of 100 genes' microarray expression data for 20 
    bone marrow samples: 10 from Acute Lymphoblastic Leukemia patients and 
    10 from healthy patients. It is useful to test mCSEAIntegrate function.
}
\usage{data(exprTest)}
\format{
    matrix
}
\source{
    Obtained from the leukemiasEset data package
} 
\keyword{datasets}

\name{mCSEA-package}
\alias{mCSEA-package}
\alias{mCSEA}
\docType{package}
\title{
    Methylated CpGs Set Enrichment Analysis
}
\description{
    Identification of diferentially methylated regions (DMRs) in predefined 
    regions (promoters, CpG islands...) from the human genome using
    Illumina's 450K or EPIC microarray data. Provides methods to rank CpG 
    probes based on linear models and includes plotting functions.
}
\author{
Jordi Martorell Marugán

Maintainer: Jordi Martorell Marugán<jordi.martorell@genyo.es>
}

\examples{
\dontrun{
library(mCSEA)
data(mcseadata)
myRank <- rankProbes(betaTest, phenoTest, refGroup = "Control")
myResults <- mCSEATest(myRank, regionsTypes = "promoters", platform = "EPIC")
}
data(precomputedmCSEA)
head(myResults$promoters)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mCSEATest.R
\name{mCSEATest}
\alias{mCSEATest}
\title{mCSEA core analysis}
\usage{
mCSEATest(rank, methData, pheno = NULL, column = 1,
  regionsTypes = c("promoters", "genes", "CGI"), customAnnotation = NULL,
  minCpGs = 5, nproc = 1, nperm = NULL, platform = "450k")
}
\arguments{
\item{rank}{A named numeric vector with the ranking statistic of each CpG
site}

\item{methData}{A data frame or a matrix containing Illumina's CpG probes in
rows and samples in columns. A SummarizedExperiment object can be used too}

\item{pheno}{A data frame or a matrix containing samples in rows and
covariates in columns. If NULL (default), pheno is extracted from the
SummarizedExperiment object}

\item{column}{The column name or number from pheno used to split the samples
into groups (first column is used by default)}

\item{regionsTypes}{A character or character vector indicating the predefined
regions to be analyzed. NULL to skip this step and use customAnnotation.}

\item{customAnnotation}{An optional list with the CpGs associated to each
feature (default = NULL)}

\item{minCpGs}{Minimum number of CpGs associated to a region. Regions below
this threshold are not tested}

\item{nproc}{Number of processors to use in GSEA step (default = 1)}

\item{nperm}{(deprecated) Number of permutations to do in GSEA step in the 
previous implementation. Now, this parameter is ignored}

\item{platform}{Platform used to get the methylation data (450k or EPIC)}
}
\value{
A list with the results of each of the analyzed regions. For each
region type, a data frame with the results and a list with the probes
associated to each region are generated. In addition, this list also contains
the input methData, pheno and platform objects
}
\description{
Perform a methylated CpG sites enrichment analysis in predefined genomic
regions
}
\examples{
\dontrun{
library(mCSEAdata)
data(mcseadata)
myRank <- rankProbes(betaTest, phenoTest, refGroup = "Control")
set.seed(123)
myResults <- mCSEATest(myRank, betaTest, phenoTest,
regionsTypes = "promoters", platform = "EPIC")
}
data(precomputedmCSEA)
head(myResults[["promoters"]])
head(myResults[["promoters_association"]])
}
\references{
Subramanian, A. et al (2005). \emph{Gene set enrichment analysis:
 A knowledge-based approach for interpreting genome-wide expression profiles}
 . PNAS 102, 15545-15550.
}
\seealso{
\code{\link{rankProbes}}, \code{\link{mCSEAPlot}},
\code{\link{mCSEAPlotGSEA}}
}
\author{
Jordi Martorell Marugán, \email{jordi.martorell@genyo.es}
}
